@@ -11,11 +11,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>	 /* for tolower(), toupper() */
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
