@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -523,22 +518,24 @@ int findpageobj(objectptr pobj)
    return -1;
 }
 
-/*-------------------------------------------------------*/
-/* Recursively find all sub-circuits associated with the */
-/* top-level circuit and set their filenames to be the	 */
-/* same.						 */
-/*							 */
-/* Avoid possible recursion problems by limiting the	 */
-/* number of recursion levels.  Presumably no circuit	 */
-/* would have more than several hundred hierarchical 	 */
-/* levels.						 */
-/*-------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Enumerate all of the pages which are subschematics   */
+/* of "toppage" and return the result in the list	*/
+/* passed as a pointer.  The list is assumed to be	*/
+/* already allocated, and equal to the total number of	*/
+/* pages (xobjs.pages).					*/
+/*							*/
+/* Avoid possible recursion problems by limiting the	*/
+/* number of recursion levels.  Presumably no circuit	*/
+/* would have more than several hundred hierarchical 	*/
+/* levels.						*/
+/*------------------------------------------------------*/
 
-int findsubschems(int toppage, objectptr cschem, int level)
+int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 {
    genericptr *cgen;
 
-   if (level == HIERARCHY_LIMIT) return -1;
+   if (level == HIERARCHY_LIMIT) return -1;	/* sanity check */
 
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
       if ((*cgen)->type == OBJECT) {
@@ -546,32 +543,22 @@ int findsubschems(int toppage, objectptr cschem, int level)
 
 	 if (cobj->symschem != NULL) {
 	    int pageno = findpageobj(cobj->symschem);
-	    /* Fprintf(stdout, "Symbol %s has schematic at page %d\n",
-			cobj->name, pageno); */
-	    if (pageno < 0) {	/* This shouldn't happen (hopefully) */
-	       Wprintf("Error: disconnected schematic!\n");
-	    }
-	    else {
-	       if (xobjs.pagelist[pageno]->filename != NULL)
-		  free(xobjs.pagelist[pageno]->filename);
-	       xobjs.pagelist[pageno]->filename =
-			strdup(xobjs.pagelist[toppage]->filename);
-	    }
+	    if ((pageno >= 0) && (pageno < xobjs.pages))
+		pagelist[pageno]++;
 
 	    /* A symbol on its own schematic page is allowed for clarity */
 	    /* of the schematic, but this cannot be a functional part of */
 	    /* the schematic circuit!					 */
 	    
 	    if (cobj->symschem != cschem) {
-	       /* Fprintf(stdout, "Searching for dependencies of symbol %s\n",
-			cobj->name); */
-	       if (findsubschems(toppage, cobj->symschem, level + 1) == -1)
+	       if (findsubschems(toppage, cobj->symschem,
+				level + 1, pagelist) == -1)
 		  return -1;
 	    }
 	 }
 	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
 	    /* Check symbols acting as their own schematics */
-	    if (findsubschems(toppage, cobj, level + 1) == -1)
+	    if (findsubschems(toppage, cobj, level + 1, pagelist) == -1)
 	       return -1;
 	 }
       }
@@ -579,6 +566,40 @@ int findsubschems(int toppage, objectptr cschem, int level)
    return 0;
 }
 
+/*-------------------------------------------------------*/
+/* Recursively find all sub-circuits associated with the */
+/* top-level circuit and set their filenames to be the	 */
+/* same.						 */
+/*-------------------------------------------------------*/
+
+void collectsubschems(int toppage)
+{
+   int level = 0;
+   objectptr cschem;
+   short *pagelist, pageno;
+
+   if (xobjs.pagelist[toppage]->pageinst == NULL) return;
+
+   cschem = xobjs.pagelist[toppage]->pageinst->thisobject;
+
+   pagelist = (short *)malloc(xobjs.pages * sizeof(short));
+
+   for (pageno = 0; pageno < xobjs.pages; pageno++)
+      pagelist[pageno] = 0;
+
+   findsubschems(toppage, cschem, 0, pagelist);
+
+   for (pageno = 0; pageno < xobjs.pages; pageno++) {
+      if (pagelist[pageno] > 0) {
+	  if (xobjs.pagelist[pageno]->filename != NULL)
+	     free(xobjs.pagelist[pageno]->filename);
+	  xobjs.pagelist[pageno]->filename =
+	     strdup(xobjs.pagelist[toppage]->filename);
+      }
+   } 
+   free((char *)pagelist);
+}
+
 /*--------------------------------------------------------*/
 /* Copy label to corresponding schematic/symbol		  */
 /*--------------------------------------------------------*/
